@@ -1,20 +1,28 @@
 from time import sleep as slp
 from sys import exit
+import requests
+import json
 from json import dumps, load
 from pathlib import Path
 from threading import Thread
+# from concurrent.futures import ThreadPoolExecutor
 from contextlib import suppress
 from unicodedata import normalize
 from string import punctuation
-from random import choice
+from urllib.request import urlopen
 from datetime import datetime
-from amino import Client, SubClient, ACM
+
+from .local_amino import Client, SubClient, ACM
+
 from uuid import uuid4
-# this is the Slimakoi's API with some of my patches
+from inspect import getfullargspec
+
+# this is Slimakoi's API with some of my patches
 
 # API made by ThePhoenix78
 # Big optimisation thanks to SempreLEGIT#1378 ♥
-
+# small very small changes by meliodas
+# if login method is not working use sid
 
 path_utilities = "utilities"
 path_amino = f'{path_utilities}/amino_list'
@@ -37,12 +45,19 @@ class Command:
         self.conditions = {}
 
     def execute(self, commande, data, type: str = "command"):
-        if commande in self.conditions[type].keys():
+        com = self.commands[type][commande]
+        arg = getfullargspec(com).args
+        arg.pop(0)
+        s = len(arg)
+        dico = {}
+        if s:
+            dico = {key: value for key, value in zip(arg, data.message.split()[0:s])}
+
+        if self.conditions[type].get(commande, None):
             if self.conditions[type][commande](data):
-                return self.commands[type][commande](data)
-            else:
-                return
-        return self.commands[type][commande](data)
+                return self.commands[type][commande](data, **dico)
+            return
+        return self.commands[type][commande](data, **dico)
 
     def categorie_exist(self, type: str):
         return type in self.commands.keys()
@@ -75,7 +90,6 @@ class Command:
             if callable(condition):
                 for command in name:
                     self.conditions[type][command] = condition
-
             for command in name:
                 self.commands[type][command.lower()] = command_funct
             return command_funct
@@ -225,7 +239,10 @@ class BannedWords:
 
 
 class Parameters:
-    __slots__ = ("subClient", "chatId", "authorId", "author", "message", "messageId", "comId", "info")
+    __slots__ = (
+                    "subClient", "chatId", "authorId", "author", "message", "messageId",
+                    "authorIcon", "comId", "replySrc", "replyMsg", "replyId", "info"
+                 )
 
     def __init__(self, data, subClient):
         self.subClient = subClient
@@ -234,14 +251,27 @@ class Parameters:
         self.author = data.message.author.nickname
         self.message = data.message.content
         self.messageId = data.message.messageId
+        self.authorIcon = data.message.author.icon
         self.comId = data.comId
+
+        self.replySrc = None
+        self.replyId = None
+        if data.message.extensions and data.message.extensions.get('replyMessage', None) and data.message.extensions['replyMessage'].get('mediaValue', None):
+            self.replySrc = data.message.extensions['replyMessage']['mediaValue'].replace('_00.', '_hq.')
+            self.replyId = data.message.extensions['replyMessage']['messageId']
+
+        self.replyMsg = None
+        if data.message.extensions and data.message.extensions.get('replyMessage', None) and data.message.extensions['replyMessage'].get('content', None):
+            self.replyMsg = data.message.extensions['replyMessage']['content']
+            self.replyId = data.message.extensions['replyMessage']['messageId']
+
         self.info = data
 
 
 class BotAmino(Command, Client, TimeOut, BannedWords):
-    def __init__(self, email: str = None, password: str = None, sid: str = None,  proxies: dict = None):
+    def __init__(self, email: str = None, password: str = None, sid: str = None,  proxies: dict = None, deviceId: str = "2271017D5F917B37DAC9C325B10542BC9B63109292D882729D1813D5355404380E2F1A699A34629C10", certificatePath: str = None):
         Command.__init__(self)
-        Client.__init__(self, proxies=proxies)
+        Client.__init__(self, proxies=proxies, deviceId="2271017D5F917B37DAC9C325B10542BC9B63109292D882729D1813D5355404380E2F1A699A34629C10", certificatePath=certificatePath)
 
         if email and password:
             self.login(email=email, password=password)
@@ -270,6 +300,8 @@ class BotAmino(Command, Client, TimeOut, BannedWords):
         self.self_callable = False
         self.no_command_message = ""
         self.spam_message = "You are spamming, be careful"
+        self.lock_message = "Command locked sorry"
+        self.launched = False
 
     def tradlist(self, sub):
         sublist = []
@@ -281,6 +313,12 @@ class BotAmino(Command, Client, TimeOut, BannedWords):
             sublist.append(elem)
         return sublist
 
+    def send_data(self, data):
+        self.send(data)
+
+    def add_community(self, comId):
+        self.communaute[comId] = Bot(self, comId, self.prefix, self.bio, self.activity)
+
     def get_community(self, comId):
         return self.communaute[comId]
 
@@ -296,6 +334,106 @@ class BotAmino(Command, Client, TimeOut, BannedWords):
     def generate_transaction_id(self):
         return str(uuid4())
 
+    def start_screen_room(self, comId: str, chatId: str, joinType: int=1):
+        data = {
+            "o": {
+                "ndcId": comId,
+                "threadId": chatId,
+                "joinRole": joinType,
+                "id": "2154531"  # Need to change?
+            },
+            "t": 112
+        }
+        data = dumps(data)
+        self.send(data)
+
+        data = {
+            "o": {
+                "ndcId": int(comId),
+                "threadId": chatId,
+                "joinRole": joinType,
+                "channelType": 5,
+                "id": "2154531"  # Need to change?
+            },
+            "t": 108
+        }
+        data = dumps(data)
+        self.send(data)
+
+    def join_screen_room(self, comId: str, chatId: str, joinType: int=1):
+        data = {
+            "o":
+                {
+                    "ndcId": int(comId),
+                    "threadId": chatId,
+                    "joinRole": 2,
+                    "id": "72446"
+                },
+            "t": 112
+        }
+        data = dumps(data)
+        self.send(data)
+
+    def start_voice_room(self, comId: str, chatId: str, joinType: int=1):
+        data = {
+            "o": {
+                "ndcId": comId,
+                "threadId": chatId,
+                "joinRole": joinType,
+                "id": "2154531"  # Need to change?
+            },
+            "t": 112
+        }
+        data = dumps(data)
+        self.send(data)
+        data = {
+            "o": {
+                "ndcId": comId,
+                "threadId": chatId,
+                "channelType": 1,
+                "id": "2154531"  # Need to change?
+            },
+            "t": 108
+        }
+        data = dumps(data)
+        self.send(data)
+
+    def end_voice_room(self, comId: str, chatId: str, joinType: int = 2):
+        data = {
+            "o": {
+                "ndcId": comId,
+                "threadId": chatId,
+                "joinRole": joinType,
+                "id": "2154531"  # Need to change?
+            },
+            "t": 112
+        }
+        data = dumps(data)
+        self.send(data)
+
+    def show_online(self, comId):
+        data = {
+            "o": {
+                "actions": ["Browsing"],
+                "target": f"ndc://x{comId}/",
+                "ndcId": int(comId),
+                "id": "82333"
+            },
+            "t":304}
+        data = dumps(data)
+        slp(2)
+        self.send(data)
+
+    def upload_bubble(self,file,comId):
+        data=file
+        response = requests.post(f"https://service.narvii.com/api/v1/x{comId}/s/chat/chat-bubble/templates/107147e9-05c5-405f-8553-af65d2823457/generate", data=data, headers=self.headers)
+        bid=json.loads(response.text)['chatBubble']['bubbleId']
+        print(bid)
+        response = requests.post(f"https://service.narvii.com/api/v1/x{comId}/s/chat/chat-bubble/{bid}", data=data, headers=self.headers)
+        if response.status_code !=200:
+            return json.loads(response.text)
+        else: return bid
+
     def check(self, args, *can, id_=None):
         id_ = id_ if id_ else args.authorId
         foo = {'staff': args.subClient.is_in_staff,
@@ -305,8 +443,16 @@ class BotAmino(Command, Client, TimeOut, BannedWords):
             if foo[i](id_):
                 return True
 
-    def threadLaunch(self, commu, passive):
-        self.communaute[commu] = Bot(self, commu, self.prefix, self.bio, self.activity)
+    def check_all(self):
+        amino_list = self.sub_clients()
+        for com in amino_list.comId:
+            try:
+                self.communaute[com].check_in()
+            except Exception:
+                pass
+
+    def threadLaunch(self, commu, passive: bool=False):
+        self.communaute[commu] = Bot(self, commu, self.prefix, self.bio, passive)
         slp(30)
         if passive:
             self.communaute[commu].passive()
@@ -316,6 +462,9 @@ class BotAmino(Command, Client, TimeOut, BannedWords):
         self.len_community = len(amino_list.comId)
         [Thread(target=self.threadLaunch, args=[commu, passive]).start() for commu in amino_list.comId]
 
+        if self.launched:
+            return
+
         if self.categorie_exist("command") or self.categorie_exist("answer"):
             self.launch_text_message()
 
@@ -337,6 +486,39 @@ class BotAmino(Command, Client, TimeOut, BannedWords):
         if self.categorie_exist("on_all"):
             self.launch_all_message()
 
+        self.launched = True
+
+    def single_launch(self, commu, passive: bool = False):
+        amino_list = self.sub_clients()
+        self.len_community = len(amino_list.comId)
+        Thread(target=self.threadLaunch, args=[commu, passive]).start()
+
+        if self.launched:
+            return
+
+        if self.categorie_exist("command") or self.categorie_exist("answer"):
+            self.launch_text_message()
+
+        if self.categorie_exist("on_member_join_chat"):
+            self.launch_on_member_join_chat()
+
+        if self.categorie_exist("on_member_leave_chat"):
+            self.launch_on_member_leave_chat()
+
+        if self.categorie_exist("on_other"):
+            self.launch_other_message()
+
+        if self.categorie_exist("on_remove"):
+            self.launch_removed_message()
+
+        if self.categorie_exist("on_delete"):
+            self.launch_delete_message()
+
+        if self.categorie_exist("on_all"):
+            self.launch_all_message()
+
+        self.launched = True
+
     def message_analyse(self, data, type):
         try:
             commuId = data.comId
@@ -346,6 +528,17 @@ class BotAmino(Command, Client, TimeOut, BannedWords):
 
         args = Parameters(data, subClient)
         Thread(target=self.execute, args=[type, args, type]).start()
+        """
+        try:
+            with ThreadPoolExecutor() as ex:
+                th = ex.submit(self.execute, type, args, type)
+                val = th.result()
+            if val:
+                return val
+            return
+        except Exception:
+            Thread(target=self.execute, args=[type, args, type]).start()
+        """
 
     def on_member_event(self, data, type):
         try:
@@ -382,6 +575,11 @@ class BotAmino(Command, Client, TimeOut, BannedWords):
             elif "command" in self.commands.keys() and args.message.startswith(subClient.prefix) and not self.check(args, "bot"):
                 print(f"{args.author} : {args.message}")
                 command = args.message.lower().split()[0][len(subClient.prefix):]
+
+                if command in subClient.locked_command:
+                    subClient.send_message(args.chatId, self.lock_message)
+                    return
+
                 args.message = ' '.join(args.message.split()[1:])
                 self.time_user(args.authorId, self.wait)
                 if command.lower() in self.commands["command"].keys():
@@ -526,33 +724,31 @@ class Bot(SubClient, ACM):
 
         self.update_file(old_dict)
 
-        self.subclient = SubClient(comId=self.community_id, profile=client.profile)
+        # self.subclient = SubClient(comId=self.community_id, profile=client.profile)
 
         self.banned_words = self.get_file_info("banned_words")
+        self.locked_command = self.get_file_info("locked_command")
         self.message_bvn = self.get_file_info("welcome")
         self.welcome_chat = self.get_file_info("welcome_chat")
         self.prefix = self.get_file_info("prefix")
         self.favorite_users = self.get_file_info("favorite_users")
         self.favorite_chats = self.get_file_info("favorite_chats")
         self.update_file()
-        self.activity_status("on")
+        #self.activity_status("on")
         new_users = self.get_all_users(start=0, size=30, type="recent")
 
         self.new_users = [elem["uid"] for elem in new_users.json["userProfileList"]]
 
-        if self.welcome_chat or self.message_bvn:
-            Thread(target=self.check_new_member).start()
-
     def create_community_file(self):
         with open(f'{path_amino}/{self.community_amino_id}.json', 'w', encoding='utf8') as file:
             dict = self.create_dict()
             file.write(dumps(dict, sort_keys=False, indent=4))
 
     def create_dict(self):
-        return {"welcome": "", "prefix": self.prefix, "welcome_chat": "", "favorite_users": [], "favorite_chats": [], "banned_words": []}
+        return {"welcome": "", "prefix": self.prefix, "welcome_chat": "", "locked_command": [], "favorite_users": [], "favorite_chats": [], "banned_words": []}
 
     def get_dict(self):
-        return {"welcome": self.message_bvn, "prefix": self.prefix, "welcome_chat": self.welcome_chat,
+        return {"welcome": self.message_bvn, "prefix": self.prefix, "welcome_chat": self.welcome_chat, "locked_command": self.locked_command,
                 "favorite_users": self.favorite_users, "favorite_chats": self.favorite_chats, "banned_words": self.banned_words}
 
     def update_file(self, dict=None):
@@ -596,6 +792,10 @@ class Bot(SubClient, ACM):
         self.banned_words.extend(liste)
         self.update_file()
 
+    def add_locked_command(self, liste: list):
+        self.locked_command.extend(liste)
+        self.update_file()
+
     def remove_favorite_users(self, value: str):
         liste = [value]
         [self.favorite_users.remove(elem) for elem in liste if elem in self.favorite_users]
@@ -610,6 +810,10 @@ class Bot(SubClient, ACM):
         [self.banned_words.remove(elem) for elem in liste if elem in self.banned_words]
         self.update_file()
 
+    def remove_locked_command(self, liste: list):
+        [self.locked_command.remove(elem) for elem in liste if elem in self.locked_command]
+        self.update_file()
+
     def unset_welcome_chat(self):
         self.welcome_chat = ""
         self.update_file()
@@ -717,6 +921,27 @@ class Bot(SubClient, ACM):
                 return chat_id
         return False
 
+    def copy_bubble(self,chatId:str,replyId:str,comId:str):
+        header={
+    'Accept-Language': 'en-US',
+    'Content-Type': 'application/octet-stream',
+    'User-Agent': 'Dalvik/2.1.0 (Linux; U; Android 7.1; LG-UK495 Build/MRA58K; com.narvii.amino.master/3.3.33180)',
+    'Host': 'service.narvii.com',
+    'Accept-Encoding': 'gzip',
+    'Connection': 'Keep-Alive',
+    }
+        a=self.get_message_info(chatId=chatId,messageId=replyId).json["chatBubble"]["resourceUrl"]
+        print(a)
+        with urlopen(a) as zipresp:
+            yo=zipresp.read()
+        print(yo)
+        response = requests.post(f"https://service.narvii.com/api/v1/x{comId}/s/chat/chat-bubble/templates/107147e9-05c5-405f-8553-af65d2823457/generate", data=yo, headers=header)
+        bid=json.loads(response.text)['chatBubble']['bubbleId']
+        print(bid)
+        response = requests.post(f"https://service.narvii.com/api/v1/{comId}/s/chat/chat-bubble/{bid}", data=yo, headers=header)
+        print("uploaded")
+
+
     def stop_instance(self):
         self.marche = False
 
@@ -724,12 +949,12 @@ class Bot(SubClient, ACM):
         self.marche = True
         Thread(target=self.passive).start()
 
-    def leave_community(self):
-        self.client.leave_community(comId=self.community_id)
+    def leave_amino(self):
         self.marche = False
         for elem in self.get_public_chat_threads().chatId:
             with suppress(Exception):
                 self.leave_chat(elem)
+        self.client.leave_community(comId=self.community_id)
 
     def check_new_member(self):
         if not (self.message_bvn or self.welcome_chat):
@@ -747,7 +972,8 @@ class Bot(SubClient, ACM):
 
             if not val and self.welcome_chat:
                 with suppress(Exception):
-                    self.send_message(chatId=self.welcome_chat, message=f"Welcome here ‎‏‎‏@{name}!‬‭", mentionUserIds=[uid])
+                    self.invite_to_chat(chatId=self.welcome_chat, userId=uid)
+                    self.invite_to_chat(chatId="cce71e6d-c2db-40d8-a7b2-41f47b713f97",userId=uid)
 
         new_users = self.get_all_users(start=0, size=30, type="recent")
         self.new_users = [elem["uid"] for elem in new_users.json["userProfileList"]]
@@ -767,7 +993,8 @@ class Bot(SubClient, ACM):
 
             if uid not in self.new_users and self.welcome_chat:
                 with suppress(Exception):
-                    self.send_message(chatId=self.welcome_chat, message=f"Welcome here ‎‏‎‏@{name}!‬‭", mentionUserIds=[uid])
+                    self.invite_to_chat(chatId=self.welcome_chat, userId=uid)
+                    self.invite_to_chat(chatId="cce71e6d-c2db-40d8-a7b2-41f47b713f97",userId=uid)
 
         new_users = self.get_all_users(start=0, size=30, type="recent")
         self.new_users = [elem["uid"] for elem in new_users.json["userProfileList"]]
@@ -834,6 +1061,16 @@ class Bot(SubClient, ACM):
 
         return False
 
+    def start_screen_room(self, chatId: str, joinType: int=1):
+        self.client.join_video_chat(comId=self.community_id, chatId=chatId, joinType=joinType)
+
+    def start_voice_room(self, chatId: str, joinType: int=1):
+        self.client.join_voice_chat(comId=self.community_id, chatId=chatId, joinType=joinType)
+
+    def join_screen_room(self, chatId: str, joinType: int=1):
+        self.client.join_video_chat_as_viewer(comId=self.community_id, chatId=chatId, joinType=joinType)
+
+
     def get_chats(self):
         return self.get_public_chat_threads()
 
@@ -880,25 +1117,11 @@ class Bot(SubClient, ACM):
     def passive(self):
         def upt_activity():
             timeNow = int(datetime.timestamp(datetime.now()))
-            timeEnd = timeNow + 300
-            try:
-                self.send_active_obj(startTime=timeNow, endTime=timeEnd)
-            except Exception:
-                pass
+            timeEnd = timeNow + 30
 
         def change_bio_and_welcome_members():
             if self.welcome_chat or self.message_bvn:
                 Thread(target=self.welcome_new_member).start()
-            try:
-                self.activity_status('on')
-                if isinstance(self.bio_contents, list):
-                    self.edit_profile(content=choice(self.bio_contents))
-
-                elif isinstance(self.bio_contents, str):
-                    self.edit_profile(content=self.bio_contents)
-
-            except Exception as e:
-                print_exception(e)
 
         def feature_chats():
             try:
@@ -926,8 +1149,9 @@ class Bot(SubClient, ACM):
                 feature_users()
                 k = 0
 
-            slp(30)
             if self.activity:
                 upt_activity()
+
+            slp(30)
             j += 1
             k += 1
